 /**
  * Created by Gifer on 24.05.2016.
  */

 "use strict";

 const electron = require('electron');
 const console  = require('gifer-console');

 console.SetLogLevel(console.logLevel.L_Full);
 console.SetDropTime(-1);

 const CMD      = require('./www/js/cmd');
 const Service  = require('./service');

 const clientConnect = require('./connectToClient');

 const {app, BrowserWindow, ipcMain} = electron;

 let electronCMD  = {};
 let mcConnect    = null;
 let mainWindow   = null;
 let clientConfig = {
     configFile : {
         port       : 3333,
         room       : "36",   // MID
         video      : true,
         title      : 'title',
         uins       : '1,2,3,4',
         send_display    : false,
         receive_display : false,
 
         iplist     : '',
         turnport   : 8888,
         credential : 'sfsdfsdf',
         username   : 'dfgdfg',
         transport  : 'tcp',

         // titles
         titleaudio : "Audio Call",
         audiooff   : "Turn off microphone",
         audioon    : "Turn on microphone",
         videooff   : "Turn off camera",
         videoon    : "Turn on camera",
         hungup     : "Hung up call",

         iceTransportPolicy : "all",
         transportUDP : false,
         transportTCP : false
     },

     iceServers   : []
 };
 let getDevicesList = process.argv[1] === "getDevicesList";
 let startPlugin    = process.argv[1] === "start";
 let __closed       = false;
 let _version       = "17.09.2019";

 console.DublicateToFile('../logs/' + Service.formatDate(new Date(), 'yyyy/mm/dd') + '/nodewebkit.log');

 console.log('=====================================================');
 console.log('=========  Start Media Plugin - ' + _version + ' ==========');
 console.log('=====================================================');

 function closeWindow(msg, code){
     if (!__closed){
         __closed = true;

         try {
             if (msg === CMD.END){
                 console.log('Close Window');
             } else
             if (msg === CMD.CLOSE){
                 console.log('Close Window, msg:' + msg + (code ? " with EXIT CODE: " + code : ""));
                 console.log('Send close info: ' + msg);

                 if (mcConnect && mcConnect.SendDataToServer) {
                     mcConnect.SendDataToServer(msg);
                 }
             } else
             if (msg) { // error message
                 console.log('Close Window, msg:' + msg + (code ? " with EXIT CODE: " + code : ""));
                 console.log('ERROR: ' + msg);

                 if (mcConnect && mcConnect.SendDataToServer) {
                     mcConnect.SendDataToServer('error=' + msg);
                 }
             }
         } catch (e){}

         if (msg !== CMD.END){
             setTimeout(function(){
                 if (code){
                     app.exit(code);
                 } else {
                     app.quit();
                 }
             }, 2);
         }
     }
 }

 function loadAndSetHandlers() {
     mainWindow.loadURL(`file://${__dirname}` + (getDevicesList || !startPlugin ? '/www/getdeviceslist.html' : '/www/index.html'));

     mainWindow.on('close', () => {
         closeWindow(CMD.END);

         mainWindow.hide();
     });

     mainWindow.on('closed', () => {
         mainWindow = null;

         app.quit();
     });
 }

 function createWindow() {
     if (getDevicesList) {
         mainWindow = new BrowserWindow({
             width       : 1,
             height      : 1,
             show        : false,
             center      : true,
             alwaysOnTop : false,
             skipTaskbar : true,
             title       : "",
             autoHideMenuBar: true,
             frame       : false,
             transparent : true
         });
     } else
     if (!startPlugin){
         mainWindow = new BrowserWindow({
             height      : 300,
             show        : false,
             center      : true,
             alwaysOnTop : true,
             skipTaskbar : true,
             title       : "",
             autoHideMenuBar: true,
             frame       : true
         });
     }

     if (getDevicesList || !startPlugin) {
         loadAndSetHandlers();
         
         mainWindow.show();
     } else {
         Service.LoadFile('options.json', function (data) {
             clientConfig.configFile = Service.stringToObj(data);

             mainWindow = new BrowserWindow({
                 width       : 229,
                 height      : 79,
                 show        : false,
                 center      : true,
                 alwaysOnTop : true,
                 skipTaskbar : true,
                 title       : "",
                 autoHideMenuBar: true,
                 frame       : false,
                 transparent : clientConfig.configFile.transparent
             });

             loadAndSetHandlers();
             
             let ipList = clientConfig.configFile.iplist.split(',');

             clientConfig.configFile.iplist = ipList;

             ipList.forEach(function(ip){
                 let tcp  = 'turn:' + ip + ':' + clientConfig.configFile.turnport + '?transport=tcp';
                 let udp  = 'turn:' + ip + ':' + clientConfig.configFile.turnport + '?transport=udp';
                 let stun = 'stun:' + ip + ':' + clientConfig.configFile.turnport;

                 clientConfig.iceServers.push({
                     urls       : []
                         .concat(clientConfig.configFile.transportUDP ? udp : [])
                         .concat(clientConfig.configFile.transportTCP ? tcp : [])
                         .concat(stun),

                     credential : clientConfig.configFile.credential,
                     username   : clientConfig.configFile.username
                 });
             });

             mainWindow.show();

             mainWindow.on('crashed', function () {
                 console.err('PLUGIN CRASHED! :(');

                 closeWindow('CRASHED!', 2);
             });
         });
     }

    // mainWindow.openDevTools();
 }

 // === IMPLEMENTATION ===============================================

 app.on('ready', createWindow);

 ipcMain.on(CMD.ce_CMD, (res, data) => {
     let cmd = data.shift();

     if (electronCMD.hasOwnProperty(cmd)){
         electronCMD[cmd].apply(res, data);
     }
 });

 // ==================================================================

 electronCMD[CMD.media_error] = closeWindow;

 electronCMD[CMD.CLOSE]       = () => closeWindow(CMD.CLOSE);

 electronCMD[CMD.sendToClient] = (data) => {
     mcConnect.SendDataToServer(data);
 };

 electronCMD[CMD.START] = () => {
     console.log('Try connecting to MyChat Client');
     console.log('Host: 127.0.0.1:' + clientConfig.configFile.port);

     setTimeout(() => {
         try {
             mcConnect = new clientConnect(
                 clientConfig.configFile,                                      // config
                 (...data)   => mainWindow.webContents.send(CMD.ec_CMD, data), // cmd parser
                 (err, code) => closeWindow(err || CMD.CLOSE, code)            // disconnect callback
             );
         } catch (e){
             closeWindow('Failed connection to MyChat Client', 2);
         }
     }, 100);
 };

 // === Internal ===========================

 electronCMD[CMD.ce_get_config] = () => {
     mainWindow.webContents.send(CMD.ec_CMD, [
         CMD.ec_get_config,
         clientConfig
     ]);
 };

 electronCMD[CMD.ce_console_log] = (log, msg) => {
     console.important(msg);
 };

 electronCMD[CMD.ce_set_video_size] = (w, h, noResizable) => {
     mainWindow.setSize(w, h);

     if (noResizable){
         mainWindow.setResizable(false);
     }
 };

 electronCMD[CMD.ce_full_screen] = () => {
     mainWindow.setFullScreen(true);
 };

 electronCMD[CMD.ce_devices_list] = (devicesList) => {
     if (getDevicesList || startPlugin){
         Service.addLineToFile(Service.convertToEntities(devicesList), `${Service.ExtractPath(process.argv[0])}/devicelist.json`, true);

         mainWindow.close();
     }
 };

 electronCMD[CMD.ce_normal_screen] = () => mainWindow.setFullScreen(false);

 process.on('uncaughtException', function (error) {
     console.err(error.message || error);

     closeWindow('CRASHED! "uncaughtException"', 2);
 });
