 /**
 * Created by Gifer on 24.05.2016.
 */

 "use strict";

 let electronHandler = require('electron').ipcRenderer;
 let CMD             = require(__dirname + '/js/cmd.js');

 let ___console = window.console;
 let console = {
     log(){
         ___console.log.apply(___console, arguments);

         internalSender([
             CMD.ce_console_log,
             'log',
             arguments[0]
         ]);
     },
     error(){
         ___console.error.apply(___console, arguments);

         internalSender([
             CMD.ce_console_log,
             'err',
             arguments[0]
         ]);
     },
     err(){
         ___console.error.apply(___console, arguments);

         internalSender([
             CMD.ce_console_log,
             'err',
             arguments[0]
         ]);
     }
 };

 let ProcessCMD   = {};
 let SendCMD      = {};
 let clientConfig = {};
 let IItems       = {};

 let CallMediaEvents = new CallStates();
 let listCE          = CallMediaEvents.list;

 let mediaCall  = null;
 let mediaTimer = null;

 let isFullScreen = false;
 let isFit        = false;
 let panelShowed  = false;
 let hideTimer    = null;
 let fadeControl  = null;

 function Timer (_target){
     let Now = 0;
     let id = null;

     this.Start = function(){
         console.log('Start timer');

         id = setInterval(function(){
             Now++;

             let ss = Now % 60;
             let hh = parseInt(Now / 3600);
             let nn = parseInt(Now / 60) - (hh * 60);

             _target.innerText = ((hh < 10) ? ("0" + hh) : hh) + ":" + ((nn < 10) ? ("0" + nn) : nn) + ":" + ((ss < 10) ? ("0" + ss) : ss);
         }, 1000);
     };

     /**
      * @return {number}
      */
     this.Stop = function(){
         clearInterval(id);

         console.log('Stop timer, Now: ' + Now);

         return Now;
     };
 }

 function Fade(elem) {
     let el   = elem;
     let stop = null;

     this.in = function () {
         stop = "out";

         let last = + new Date();
         let tick = function() {
             if (stop === "in") {
                 stop = null;
             } else {
                 el.style.opacity = + el.style.opacity + (new Date() - last) / 400;

                 last = + new Date();

                 if (el.style.opacity <= 1) {
                     (window.requestAnimationFrame && requestAnimationFrame(tick)) || setTimeout(tick, 16);
                 } else {
                     stop = null;
                 }
             }
         };

         tick();
     };

     this.out = function () {
         stop = "in";
         // el.style.opacity = 1;

         let last = new Date();
         let tick = function() {
             if (stop === "out"){
                 stop = null;
             } else {
                 el.style.opacity = + el.style.opacity - (new Date() - last) / 400;

                 last = new Date();

                 if (el.style.opacity > 0) {
                     (window.requestAnimationFrame && requestAnimationFrame(tick)) || setTimeout(tick, 16);
                 } else {
                     stop = null;
                 }
             }
         };

         tick();
     };
 }

 function minMax() {
     if (mediaCall.useVideo || mediaCall.shareScreen){
         if (isFullScreen){
             SendCMD[CMD.ce_normal_screen]();

             IItems.showHideScreen.className = "";
         } else {
             SendCMD[CMD.ce_full_screen]();

             IItems.showHideScreen.className = "showHideScreenOFF";
         }

         isFullScreen = !isFullScreen;
     }
 }
 
 function initItems() {
     IItems.timer            = document.querySelector("#callTime");
     IItems.wrapCalling      = document.querySelector("#wrapCalling");
     IItems.mcWrapper        = document.querySelector("#mcWrapper");
     IItems.videoWrapper     = document.querySelector("#videoWrapper");
     IItems.cPanel           = document.querySelector("#cPanel");

     IItems.selfVideo        = document.querySelector("#mirror");
     IItems.callerVideo      = document.querySelector("#callee");
     IItems.audioStream      = document.querySelector("#audioStream");

     IItems.userName         = document.querySelector("#userName");
     IItems.cameraSwitch     = document.querySelector("#cameraSwitch");
     IItems.microphoneSwitch = document.querySelector("#microphoneSwitch");
     IItems.cancelCall       = document.querySelector("#cancelCall");

     IItems.fillFitScreen    = document.querySelector("#fillFitScreen");
     IItems.showHideScreen   = document.querySelector("#showHideScreen");

     document.addEventListener("dblclick", minMax);

     IItems.cameraSwitch.addEventListener('click', function(){
         if (mediaCall.switchCamera()){
             IItems.cameraSwitch.className = "";
         } else {
             IItems.cameraSwitch.className = "cameraSwitchOFF";
         }
     });

     IItems.microphoneSwitch.addEventListener('click', function(){
         if (mediaCall.switchMicrophone()) {
             IItems.microphoneSwitch.className = "";
         } else {
             IItems.microphoneSwitch.className = "microphoneSwitchOFF";
         }
     });

     IItems.fillFitScreen.addEventListener('click', function(){
         if (isFit) {
             IItems.fillFitScreen.className = "";
             IItems.callerVideo.className   = "";
             IItems.videoWrapper.className  = "";
         } else {
             IItems.fillFitScreen.className = "fillFitScreenOFF";
             IItems.callerVideo.className   = "fit";
             IItems.videoWrapper.className  = "center";
         }

         isFit = !isFit;
     });

     IItems.showHideScreen.addEventListener('click', minMax);

     IItems.cancelCall.addEventListener('click', () => closeCall() );
 }

 function showPanel() {
     if (panelShowed){
         clearTimeout(hideTimer);

         hideTimer = setTimeout(function () {
             panelShowed = false;
             hideTimer   = null;

             fadeControl.out();
         }, 4000);
     } else {
         fadeControl.in();

         panelShowed = true;
     }
 }

 function prepareCall(config) {
     initItems();

     mediaTimer = new Timer(IItems.timer);

     IItems.userName.innerText = config.title;
     mediaCall.useVideo        = config.video;
     mediaCall.MID             = config.room;
     mediaCall.Share           = config.send_display;
     mediaCall.shareScreen     = config.receive_display;

     mediaCall.restoreCall     = true;

     mediaCall.sendAnswer      = SendCMD[CMD.ANSWER];
     mediaCall.sendOffer       = SendCMD[CMD.OFFER];
     mediaCall.sendCandidate   = SendCMD[CMD.CANDIDATE];

     mediaCall.setMirror(IItems.selfVideo);
     mediaCall.setCallee(mediaCall.useVideo || mediaCall.shareScreen ? IItems.callerVideo : IItems.audioStream);

     mediaCall.onFailedInit = function () {
         mediaCall.iceClosed = null;

         closeCall();

         SendCMD[CMD.media_error]('Failed init connect');
     };

     mediaCall.onError   = function(err){
         console.error(err);

         mediaCall.iceClosed = null;

         CallMediaEvents.setStateEvent(listCE.outgoing_call_my_error, "outgoing_call_my_error");

         closeCall();

         SendCMD[CMD.media_error](err);
     };

     mediaCall.iceClosed = SendCMD[CMD.CLOSE];

     mediaCall.showMedia = function(){ // calleeVideoDIV
         if (!mediaCall.complete){
             mediaTimer.Start();
             mediaCall.complete = true;

             IItems.wrapCalling.style.display    = 'none';
             IItems.mcWrapper.style.display      = 'block';
             IItems.showHideScreen.style.display = 'none';
             IItems.cPanel.className             = "";

             if (mediaCall.useVideo || mediaCall.shareScreen){
                 SendCMD[CMD.ce_set_video_size](600, 452);

                 IItems.cPanel.className    += "whiteBG";
                 IItems.cPanel.style.opacity = 1;

                 panelShowed = true;

                 showPanel();

                 IItems.mcWrapper.onmousemove = function () {
                     showPanel();
                 };
             } else {
                 SendCMD[CMD.ce_set_video_size](229, 89, true);
             }

             if (mediaCall.useVideo){
                 IItems.selfVideo.style.display      = 'inline-block';
                 IItems.cameraSwitch.style.display   = 'inline-block';
                 IItems.showHideScreen.style.display = 'inline-block';
             } else {
                 IItems.selfVideo.style.display      = 'none';
                 IItems.cameraSwitch.style.display   = 'none';
             }

             if (mediaCall.shareScreen){
                 IItems.fillFitScreen .style.display = 'inline-block';
                 IItems.showHideScreen.style.display = 'inline-block';
             } else {
                 IItems.fillFitScreen .style.display = 'none';
             }

             document.onkeydown = function (e) {
                 if ((e.keyCode === 27 && isFullScreen) || e.keyCode === 13 || e.keyCode === 122){
                     minMax();
                 }
             };

             SendCMD[CMD.STARTTIMER]();
         }
     };

     CallMediaEvents.setUniversalStateEvent(function(msg){
         let state = this;

         console.log('Set call state: ' + msg);

         if (state !== listCE.outgoing_call_try      &&
             state !== listCE.outgoing_call_started  &&
             state !== listCE.outgoing_call_accepted &&
             state !== listCE.incoming_call_accepted &&
             state !== listCE.incoming_call_started  &&
             state !== listCE.incoming_call_try){

             CallMediaEvents.clearState();
         }
     });

     SendCMD[CMD.START]();

     fadeControl = new Fade(IItems.cPanel);
 }

 function closeCall() {
     console.log('Hangup');

     mediaCall.complete = false;

     mediaCall.hangUp();

     return mediaTimer.Stop();
 }

 function internalSender (args) {   // client => electron "ce_"
     electronHandler.send(CMD.ce_CMD, args); // args = [cmd, ...]
 }

 //=======================================================

 electronHandler.on(CMD.ec_CMD, function (e, args) { // electron => client "ec_"
     let cmdNumb = args.shift();

     if (ProcessCMD[cmdNumb]){
         ProcessCMD[cmdNumb].apply(null, args);
     } else {
         console.error(`CMD: ${cmdNumb} not found!`);
     }
 });

 //=======================================================

 ProcessCMD[CMD.ec_get_config] = (_config) => { // === START PROGRAM ===
     clientConfig = _config;

     console.log(JSON.stringify(clientConfig));

     mediaCall = new MediaCall();

     prepareCall(clientConfig.configFile);

     mediaCall.addIceServers(clientConfig.iceServers, clientConfig.configFile.iceTransportPolicy);
     mediaCall.setDevices(clientConfig.configFile);
 };

 ProcessCMD[CMD.OFFER]     = function (_data){
     CallMediaEvents.setStateEvent(listCE.incoming_call_started, "incoming_call_started");

     mediaCall.recvOffer(JSON.parse(_data));
 };

 ProcessCMD[CMD.ANSWER]    = function (_data){
     CallMediaEvents.setStateEvent(listCE.outgoing_call_started, "outgoing_call_started");

     mediaCall.recvAnswer(JSON.parse(_data));
 };

 ProcessCMD[CMD.CANDIDATE] = function (_data){
     mediaCall.recvCandidate(JSON.parse(_data));
 };                                              

 ProcessCMD[CMD.HALT]      = function (){
     console.log('HALT');

     closeCall();

     // SendCMD[CMD.CLOSE]();
 };

 ProcessCMD[CMD.READY]     = function (){
     CallMediaEvents.setStateEvent(listCE.outgoing_call_try, "outgoing_call_try");

     mediaCall.recvReady(clientConfig.configFile.uins);
 };

 ProcessCMD[CMD.SETTINGS]  = function (){ // data
     console.log('SETTINGS');
     // TODO: make settings
 };

 //=======================================================

 SendCMD[CMD.ANSWER] = (callee, mid, sdp) => {
     internalSender([
         CMD.sendToClient,
         CMD.ANSWER + "=" + JSON.stringify({
             UIN : callee,
             MID : mid,
             SDP : sdp
         })
     ]);
 };

 SendCMD[CMD.OFFER] = (callee, mid, sdp) => {
     internalSender([
         CMD.sendToClient,
         CMD.OFFER + "=" + JSON.stringify({
             UIN : callee,
             MID : mid,
             SDP : sdp
         })
     ]);
 };

 SendCMD[CMD.CANDIDATE] = (callee, mid, candidate) => {
     internalSender([
         CMD.sendToClient,
         CMD.CANDIDATE + "=" + JSON.stringify({
             UIN : callee,
             MID : mid,
             CANDIDATE : candidate
         })
     ]);
 };

 SendCMD[CMD.STARTTIMER] = () => {
     internalSender([
         CMD.sendToClient,
         CMD.STARTTIMER
     ]);
 };

 SendCMD[CMD.media_error] = (err, code) => {
     internalSender([
         CMD.media_error,
         err,
         code
     ]);
 };

 SendCMD[CMD.CLOSE] = () => internalSender([CMD.CLOSE]);

 SendCMD[CMD.START] = () => internalSender([CMD.START]);

 SendCMD[CMD.ce_normal_screen] = () => internalSender([CMD.ce_normal_screen]);

 SendCMD[CMD.ce_full_screen] = () => internalSender([CMD.ce_full_screen]);

 SendCMD[CMD.ce_set_video_size] = (...args) => internalSender([CMD.ce_set_video_size].concat(args));

 //=======================================================

 window.onload = () => {
     CallMediaEvents.setStateEvent(listCE.outgoing_call_try, "outgoing_call_try");

     internalSender([
         CMD.ce_get_config
     ]);
 };

